/*
 *      callbacks.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "listProducts.h"
#include "interface.h"
#include "callbacks.h"
#include "config.h"
#include "preferences.h"
#include "print.h"

/*** File menu ***/
void OnNew(GtkWidget *pWidget, glda *pGlda){
	if(pGlda->isSave == FALSE && pGlda->nbTotalProducts > 0){
		if(dialogYesAndNo(pGlda, "Voulez-vous enregistrer les modifications ?")){
			saveListProductsAdded(pGlda);
		}
	}
	
	pGlda->isSave = FALSE;
	if(pGlda->filesave)
		g_free(pGlda->filesave);
	pGlda->filesave = NULL;
	pGlda->nbTotalProducts = 0;
	pGlda->priceTotalProducts = 0;
	
	gtk_list_store_clear(pGlda->pTreeStoreProductsAdded);
	refreshTotalPriceAndQuantity(pGlda);
}

void OnOpen(GtkWidget *pWidget, glda *pGlda){
	if(pGlda->isSave || pGlda->nbTotalProducts == 0){
		openListProducts(pGlda);
	}else{
		if(dialogYesAndNo(pGlda, "Voulez-vous sauvegarder votre liste d'achat actuelle ?")){
			saveListProductsAdded(pGlda);
		}
		openListProducts(pGlda);
	}
}

void OnOpenRecent(GtkWidget *pWidget, glda *pGlda){
	/*gchar *pText;
	
	pText = gtk_label_get_text(GTK_LABEL(GTK_MENU_ITEM(pWidget)->label));
	
	printf("clic : %s\n", pText);*/
}

void OnSave(GtkWidget *pWidget, glda *pGlda){
	saveListProductsAdded(pGlda);
}

void OnSaveAs(GtkWidget *pWidget, glda *pGlda){
	gchar *filesavebefore;
	gboolean statebefore;
	
	if(pGlda->filesave){
		filesavebefore = g_strdup(pGlda->filesave);
		statebefore = pGlda->isSave;
		
		pGlda->filesave = NULL;
		pGlda->isSave = FALSE;
		
		saveListProductsAdded(pGlda);
		
		/* cancel */
		if(pGlda->isSave == FALSE){
			pGlda->filesave = g_strdup(filesavebefore);
			pGlda->isSave = statebefore;
		}
		g_free(filesavebefore);
	}
	/* first save */
	saveListProductsAdded(pGlda);
}

void OnPrint(GtkWidget *pWidget, glda *pGlda){
	create_pPrint(pGlda);
}

void OnDestroy(GtkWidget *pWidget, glda *pGlda){
	if(pGlda->isSave || (pGlda->isSave == FALSE && pGlda->nbTotalProducts == 0)){
		printf("Close Window...");
		gtk_main_quit();
		printf("ok\n");
	}else{
		if(saveOrNoBeforeQuit(pGlda)){
			printf("Close Window...");
			gtk_main_quit();
			printf("ok\n");
		}
	}
}

gboolean OnQuit(GtkWidget *pWindow, GdkEvent *pEvent, glda *pGlda){
	if(pGlda->isSave || (pGlda->isSave == FALSE && pGlda->nbTotalProducts == 0)){
		printf("Close Window...");
		gtk_main_quit();
		printf("ok\n");
		return FALSE;
	}else{
		if(saveOrNoBeforeQuit(pGlda)){
			printf("Close Window...");
			gtk_main_quit();
			printf("ok\n");
		}
		return TRUE;
	}
}

void OnSelectSubCategory(GtkTreeSelection *select, glda *pGlda){
	GList *listSubCategorySelected;
  gint depth;
  gint *tabIndices = NULL;
  gchar *pRefCategory = NULL;
  gchar *pRefSubCategory = NULL;
  GtkTreeIter Iter;
  GtkTreePath *pPath;
  
  /* Get the category or subcategory selected */
  listSubCategorySelected = gtk_tree_selection_get_selected_rows(select, NULL);
  
  if(listSubCategorySelected != NULL){
  	/*
  		Get the depth 
  		1 : a category is selected => clear the list of products
  		2 : a subcategory is selected => show the list of products
  	*/
  	depth = gtk_tree_path_get_depth(listSubCategorySelected->data);
  	if(depth == 1){
  		/* clear the list of products */
			gtk_list_store_clear(pGlda->pTreeStoreProducts);
  	}
  	if(depth == 2){
  		pRefCategory = (gchar*)g_malloc(3*sizeof(gchar));
    	pRefSubCategory = (gchar*)g_malloc(3*sizeof(gchar));
    	if(pRefCategory && pRefSubCategory){
	  		/* Get indices of the path */
  			tabIndices = gtk_tree_path_get_indices(listSubCategorySelected->data);
  			/* Get the path */
  			pPath = gtk_tree_path_new_from_indices(tabIndices[0], tabIndices[1], -1);
  			/* Get  the iter */
  			gtk_tree_model_get_iter((GtkTreeModel*)pGlda->pTreeStoreCategory, &Iter, pPath);
  			/* Get references of the subcategory selected */
  			gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreCategory, &Iter,
  																												REF_CAT_COLUMN, &pRefCategory,
  																												REF_SUBCAT_COLUMN, &pRefSubCategory,
  																												-1);
				create_pListProductsByReference(pGlda, pRefCategory, pRefSubCategory);
			}
			if(pRefCategory)
				g_free(pRefCategory);
			if(pRefSubCategory)
				g_free(pRefSubCategory);
  	}
  	g_list_free(listSubCategorySelected);
    free(tabIndices);
  }
}

void OnDoubleClickProduct(GtkTreeView *pTreeView, GtkTreePath *path, GtkTreeViewColumn *column, glda *pGlda){
	addProduct(pGlda);
}

void OnDoubleClickProductAdded(GtkTreeView *pTreeView, GtkTreePath *path, GtkTreeViewColumn *column, glda *pGlda){
	removeProduct(pGlda);
}

/*** Edit menu ***/
void OnAdd(GtkWidget *pWidget, glda *pGlda){
	addProduct(pGlda);
}

void OnEditQuantity(GtkCellRendererText *renderer, gchar *path, gchar *quantityInput, glda *pGlda){
	GtkTreePath *pPath;
	GtkTreeIter Iter;
	gchar *pRefCategory = NULL;
  gchar *pRefSubCategory = NULL;
  gchar *pRefProduct = NULL;
	gchar *oldPrice = NULL;
	gchar *newPrice = NULL;
	gdouble priceUnit;
	gint oldQuantity;
	gint newQuantity;
	
	newQuantity = strtol(quantityInput, NULL, 10);
	
	if (newQuantity > 0)
	{
		/* Get path */
		pPath = gtk_tree_path_new_from_string(path);
		/* Get iter */
		gtk_tree_model_get_iter((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter, pPath);
		/* Get references old price and old quantity of product */
		gtk_tree_model_get((GtkTreeModel*)pGlda->pTreeStoreProductsAdded, &Iter,
																				PRICE_PRODUCTS_ADDED_COLUMN, &oldPrice,
																				QUANTITY_PRODUCTS_ADDED_COLUMN, &oldQuantity,
																				REF_CAT_PRODUCTS_ADDED_COLUMN, &pRefCategory,
																				REF_SUBCAT_PRODUCTS_ADDED_COLUMN, &pRefSubCategory,
																				REF_PRODUCTS_ADDED_COLUMN, &pRefProduct,
																				-1);
		priceUnit = strtod(oldPrice, NULL) / oldQuantity;
		newPrice = g_strdup_printf("%.2f", priceUnit*newQuantity);
		
		/* Set the new price and new quantity */
		gtk_list_store_set(pGlda->pTreeStoreProductsAdded, &Iter,
										PRICE_PRODUCTS_ADDED_COLUMN, newPrice,
										QUANTITY_PRODUCTS_ADDED_COLUMN, newQuantity,
										-1);
		
		/* Update total number and total price */
		pGlda->nbTotalProducts = pGlda->nbTotalProducts - oldQuantity + newQuantity;
		pGlda->priceTotalProducts = pGlda->priceTotalProducts - strtod(oldPrice, NULL) + strtod(newPrice, NULL);
		
		refreshTotalPriceAndQuantity(pGlda);
		
		g_free(oldPrice);
		g_free(newPrice);
		g_free(pRefCategory);
		g_free(pRefSubCategory);
		g_free(pRefProduct);
	}
	
}

void OnRemove(GtkWidget *pWidget, glda *pGlda){
	removeProduct(pGlda);
}

void OnPreferences(GtkWidget *pWidget, glda *pGlda){
	create_pPreferences(pGlda);
}

/*** Help menu ***/
void OnAbout(GtkWidget *pWidget, glda *pGlda){
  create_pAbout(pGlda);
}
