/*
 *      listProducts.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "listProducts.h"

xmlDocPtr initListProducts(const gchar *filename){
	xmlDocPtr xmlDocListProducts = NULL;
	xmlNodePtr rootElementListProducts = NULL;
	
	/* the file exist? */
	if(g_file_test(filename, G_FILE_TEST_EXISTS) == FALSE){
    fprintf(stderr, "%s don't exist\n", filename);
    return NULL;
  }
  
  xmlDocListProducts = xmlParseFile(filename);
  if(xmlDocListProducts == NULL){
    fprintf(stderr, "%s isn't a valide XML file\n", filename);
    return NULL;
  }
  
  rootElementListProducts = xmlDocGetRootElement(xmlDocListProducts);
  if(rootElementListProducts == NULL){
    fprintf(stderr, "%s is empty\n", filename);
    return NULL;
  }
  
  return xmlDocListProducts;
}

gboolean checkCompatibilityListProducts(xmlDocPtr xmlDocListProducts, const gchar *filename){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda";
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  xmlobject = xmlXPathEval((xmlChar*)path_template, xmlperms_context);
  
  if(xmlobject->nodesetval && xmlobject->nodesetval->nodeNr == 0){
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
  	return FALSE;
  }
  
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  return TRUE;
}

gint makeListProductsCompatible(xmlDocPtr xmlDocListProducts, const gchar *filename){
	xmlDocPtr xmlDocListProductsCompatible = NULL;
	xmlNodePtr root = NULL;
	
	xmlNodePtr pCategoryNode = NULL;
	xmlNodePtr pCategoryNodeBefore = NULL;
	
	xmlNodePtr pSubCategoryNode = NULL;
	xmlNodePtr pSubCategoryNodeBefore = NULL;
	
	xmlNodePtr pProductNode = NULL;
	xmlNodePtr pProductNodeBefore = NULL;
	
	xmlNodePtr pProductNameNode = NULL;
	xmlNodePtr pProductPriceNode = NULL;
	
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  gint idCategory, idSubCategory, idProduct;
  
  gchar *pCategory = NULL;
  gchar *pRefCategory = NULL;
  
  gchar *pSubCategory = NULL;
  gchar *pRefSubCategory = NULL;
  
  gchar *pProductName = NULL;
  gchar *pProductPrice = NULL;
  gchar *pRefProduct = NULL;
  
  xmlDocListProductsCompatible = xmlNewDoc((const xmlChar*)"1.0");
  root = xmlNewNode(NULL, (const xmlChar*)"lda");
  xmlAddChild((xmlNodePtr)xmlDocListProductsCompatible, root);
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  /* check category */
  idCategory = 1;
  do{
  	path = (xmlChar*)g_strdup_printf("/liste_articles/category[%d]", idCategory);
  	xmlobject = xmlXPathEval(path, xmlperms_context);
  	
  	/* No found category */
  	if(xmlobject->nodesetval->nodeNr == 0){
  		idCategory = -1;
  		xmlFree(path);
  		path = NULL;
  		xmlXPathFreeObject(xmlobject);
  	}else{
  		pCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0],
  																			(const xmlChar*)"name"));
  		
  		pRefCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0],
  																					(const xmlChar*)"ref"));
			
  		pCategoryNode = xmlNewNode(NULL, (const xmlChar*)"category");
  		
  		if(idCategory == 1){
  			xmlAddChild(root, pCategoryNode);
  		}else{
  			xmlAddSibling(pCategoryNodeBefore, pCategoryNode);
  		}
  		
  		xmlSetProp(pCategoryNode, (const xmlChar*)"name", (const xmlChar*)pCategory);
  		xmlSetProp(pCategoryNode, (const xmlChar*)"ref", (const xmlChar*)pRefCategory);
  		
  		pCategoryNodeBefore = pCategoryNode;
  		
  		g_free(pCategory);
  		g_free(pRefCategory);
  		pCategory = NULL;
  		pRefCategory = NULL;
  		
  		g_free(path);
  		path = NULL;
  		xmlXPathFreeObject(xmlobject);
  		
  		/* check subcategory */
  		idSubCategory = 1;
  		do{
  			path = (xmlChar*)g_strdup_printf("/liste_articles/category[%d]/sous_category[%d]",
  																				idCategory, idSubCategory);
				xmlobject = xmlXPathEval(path, xmlperms_context);
				
				/* No found subcategory */
				if(xmlobject->nodesetval->nodeNr == 0){
					idSubCategory = - 1;
					xmlFree(path);
					path = NULL;
					xmlXPathFreeObject(xmlobject);
				}else{
					pSubCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0],
  																			(const xmlChar*)"name"));
  				
  				pRefSubCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0],
  																					(const xmlChar*)"ref"));
					
  				pSubCategoryNode = xmlNewNode(NULL, (const xmlChar*)"subcategory");
  				
  				if(idSubCategory == 1){
  					xmlAddChild(pCategoryNode, pSubCategoryNode);
  				}else{
  					xmlAddSibling(pSubCategoryNodeBefore, pSubCategoryNode);
  				}
  				
  				xmlSetProp(pSubCategoryNode, (const xmlChar*)"name", (const xmlChar*)pSubCategory);
  				xmlSetProp(pSubCategoryNode, (const xmlChar*)"ref", (const xmlChar*)pRefSubCategory);
  				
  				pSubCategoryNodeBefore = pSubCategoryNode;
  				
  				g_free(pSubCategory);
  				g_free(pRefSubCategory);
  				pSubCategory = NULL;
  				pRefSubCategory = NULL;
  				
  				g_free(path);
  				path = NULL;
  				xmlXPathFreeObject(xmlobject);
  				
  				/* check products */
  				idProduct = 1;
  				do{
  					path = (xmlChar*)g_strdup_printf("/liste_articles/category[%d]/sous_category[%d]/article[%d]",
  																						idCategory, idSubCategory, idProduct);
						xmlobject = xmlXPathEval(path, xmlperms_context);
						
						/* no found product */
						if(xmlobject->nodesetval->nodeNr == 0){
							idProduct = - 1;
							xmlFree(path);
							path = NULL;
							xmlXPathFreeObject(xmlobject);
						}else{
							/* Get product's reference */
							pRefProduct = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0],
  																					(const xmlChar*)"ref"));
							
							xmlFree(path);
							xmlXPathFreeObject(xmlobject);
							
							/* Get product's name */
							path = (xmlChar*)g_strdup_printf(
																					"/liste_articles/category[%d]/sous_category[%d]/article[%d]/name/text()",
  																						idCategory, idSubCategory, idProduct);
							xmlobject = xmlXPathEval(path, xmlperms_context);
							
							pProductName = g_strdup_printf("%s", (gchar *)xmlobject->nodesetval->nodeTab[0]->content);
							
							xmlFree(path);
							xmlXPathFreeObject(xmlobject);
							
							/* Get product's price */
							path = (xmlChar*)g_strdup_printf(
																					"/liste_articles/category[%d]/sous_category[%d]/article[%d]/prix/text()",
  																						idCategory, idSubCategory, idProduct);
							xmlobject = xmlXPathEval(path, xmlperms_context);
							
							pProductPrice = g_strdup_printf("%s", (gchar *)xmlobject->nodesetval->nodeTab[0]->content);
							
							xmlFree(path);
							xmlXPathFreeObject(xmlobject);
							
							pProductNode = xmlNewNode(NULL, (const xmlChar*)"product");
							
							if(idProduct == 1){
								xmlAddChild(pSubCategoryNode, pProductNode);
							}else{
								xmlAddSibling(pProductNodeBefore, pProductNode);
							}
							
							xmlSetProp(pProductNode, (const xmlChar*)"ref", (const xmlChar*)pRefProduct);
							
							pProductNameNode = xmlNewChild(pProductNode, NULL,
																						(const xmlChar *)"name", (const xmlChar *)pProductName);
  						pProductPriceNode = xmlNewChild(pProductNode, NULL,
  																					(const xmlChar *)"price", (const xmlChar *)pProductPrice);
							
							pProductNodeBefore = pProductNode;
							
							g_free(pRefProduct);
							g_free(pProductName);
							g_free(pProductPrice);
							pRefProduct = NULL;
							pProductName = NULL;
							pProductPrice = NULL;
							
							idProduct++;
						}
  				}while(idProduct != -1);
  				
  				idSubCategory++;
				}
  		}while(idSubCategory != -1);
  		
  		idCategory++;
  	}
  }while(idCategory != -1);
  
  xmlXPathFreeContext(xmlperms_context);
  
  /* save */
  if(xmlSaveFormatFileEnc(filename, xmlDocListProductsCompatible, "UTF-8", 1) == -1){
    xmlFreeDoc(xmlDocListProductsCompatible);
    return FALSE;
  }
  xmlFreeDoc(xmlDocListProductsCompatible);
  return TRUE;
}

gint getCategoryByPosition(xmlDocPtr xmlDocListProducts, gint positionCategory, gchar **pCategory, gchar **pRef){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda/category[%d]";
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  path = (xmlChar*)malloc(sizeof(path_template)+sizeof(gint));
  
  if(path == NULL){
    fprintf(stderr, "Error : Failed to allocate memory\n");
    xmlFree(path);
    xmlXPathFreeContext(xmlperms_context);
    return CATEGORY_ERROR_MALLOC;
  }
  
  sprintf((char*)path, path_template, positionCategory);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(xmlobject->nodesetval->nodeNr == 0){
    fprintf(stderr, "Error : Impossible de trouver la categorie\n");
    xmlFree(path);
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
    return CATEGORY_ERROR_NO_FOUND;
  }
  
  if(pCategory)
  	*pCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"name"));
  if(pRef)
  	*pRef = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"ref"));
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  return CATEGORY_NO_ERROR;
}

gint getSubCategoryByPosition(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gchar **pSubCategory, gchar **pRef){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda/category[%d]/subcategory[%d]";
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)malloc(sizeof(path_template)+2*sizeof(gint));
  
  if(path == NULL){
    fprintf(stderr, "Error : Failed to allocate memory\n");
    xmlFree(path);
    xmlXPathFreeContext(xmlperms_context);
    return CATEGORY_ERROR_MALLOC;
  }
  sprintf((char*)path, path_template, positionCategory, positionSubCategory);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(!(xmlobject->nodesetval) || xmlobject->nodesetval->nodeNr == 0){
    fprintf(stderr, "Error : Impossible de trouver la sous categorie\n");
    xmlFree(path);
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
    return CATEGORY_ERROR_NO_FOUND;
  }
  
  if(pSubCategory)
  	*pSubCategory = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"name"));
  if(pRef)
  	*pRef = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"ref"));
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  return CATEGORY_NO_ERROR;
}

gint getProductReferenceByPosition(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gint positionProduct, gchar **pRef){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda/category[%d]/subcategory[%d]/product[%d]";
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)malloc(sizeof(path_template)+3*sizeof(gint));
  
  if(path == NULL){
    fprintf(stderr, "Error : Failed to allocate memory\n");
    xmlFree(path);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_MALLOC;
  }
  sprintf((char*)path, path_template, positionCategory, positionSubCategory, positionProduct);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(!(xmlobject->nodesetval) || xmlobject->nodesetval->nodeNr == 0){
    fprintf(stderr, "Error : Impossible de trouver l'article\n");
    xmlFree(path);
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_NO_FOUND;
  }
  
  if(pRef)
  	*pRef = g_strdup_printf("%s", (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"ref"));
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  
  return PRODUCT_NO_ERROR;
}

gint getProductNameByPosition(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gint positionProduct, gchar **pProductName){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda/category[%d]/subcategory[%d]/product[%d]/name/text()";
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)malloc(sizeof(path_template)+3*sizeof(gint));
  
  if(path == NULL){
    fprintf(stderr, "Error : Failed to allocate memory\n");
    xmlFree(path);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_MALLOC;
  }
  sprintf((char*)path, path_template, positionCategory, positionSubCategory, positionProduct);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(!(xmlobject->nodesetval) || xmlobject->nodesetval->nodeNr == 0){
    fprintf(stderr, "Error : Impossible de trouver l'article\n");
    xmlFree(path);
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_NO_FOUND;
  }
  
  if(pProductName)
  	*pProductName = g_strdup_printf("%s", (gchar *)xmlobject->nodesetval->nodeTab[0]->content);
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  
  return PRODUCT_NO_ERROR;
}

gint getProductPriceByPosition(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gint positionProduct, gchar **pProductPrice){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  const gchar path_template[] = "/lda/category[%d]/subcategory[%d]/product[%d]/price/text()";
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)malloc(sizeof(path_template)+3*sizeof(gint));
  
  if(path == NULL){
    fprintf(stderr, "Error : Failed to allocate memory\n");
    xmlFree(path);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_MALLOC;
  }
  sprintf((char*)path, path_template, positionCategory, positionSubCategory, positionProduct);
  xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(!(xmlobject->nodesetval) || xmlobject->nodesetval->nodeNr == 0){
    fprintf(stderr, "Error : Impossible de trouver l'article\n");
    xmlFree(path);
    xmlXPathFreeObject(xmlobject);
    xmlXPathFreeContext(xmlperms_context);
    return PRODUCT_ERROR_NO_FOUND;
  }
  
  if(pProductPrice)
  	*pProductPrice = g_strdup_printf("%s", (gchar *)xmlobject->nodesetval->nodeTab[0]->content);
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
  
  return PRODUCT_NO_ERROR;
}

gint getPositionCategoryByReference(xmlDocPtr xmlDocListProducts, gchar *pRef){
	gchar *pRefCategory = NULL;
	gint positionCategory;
	gint searchCategory;
	
	positionCategory = 1;
	
	do{
		if(pRefCategory)
			g_free(pRefCategory);
		pRefCategory = NULL;
		
		searchCategory = getCategoryByPosition(xmlDocListProducts, positionCategory, NULL, &pRefCategory);
		positionCategory++;
	}while(searchCategory == CATEGORY_NO_ERROR && strcmp(pRefCategory, pRef) != 0);
	
	if(searchCategory == CATEGORY_NO_ERROR && strcmp(pRefCategory, pRef) == 0){
		g_free(pRefCategory);
		return positionCategory-1;
	}else{
		fprintf(stderr, "no found the category\n");
		return -1;
	}
}

gint getPositionSubCategoryByReference(xmlDocPtr xmlDocListProducts, gint positionCategory, gchar *pRef){
	gchar *pRefSubCategory = NULL;
	gint positionSubCategory;
	gint searchSubCategory;
	
	positionSubCategory = 1;
	
	do{
		if(pRefSubCategory)
			g_free(pRefSubCategory);
		pRefSubCategory = NULL;
		
		searchSubCategory = getSubCategoryByPosition(xmlDocListProducts,
												positionCategory, positionSubCategory,
												NULL, &pRefSubCategory);
		positionSubCategory++;
	}while(searchSubCategory == CATEGORY_NO_ERROR && strcmp(pRefSubCategory, pRef) != 0);
	
	if(searchSubCategory == CATEGORY_NO_ERROR && strcmp(pRefSubCategory, pRef) == 0){
		g_free(pRefSubCategory);
		return positionSubCategory-1;
	}else{
		fprintf(stderr, "no found the subcategory\n");
		return -1;
	}
}

gint getPositionProductByReference(xmlDocPtr xmlDocListProducts,
																	gint positionCategory, gint positionSubCategory, gchar *pRef)
{
	gchar *pRefProduct = NULL;
	gint positionProduct;
	gint searchProduct;
	
	positionProduct = 1;
	
	do{
		if(pRefProduct)
			g_free(pRefProduct);
		pRefProduct = NULL;
		
		searchProduct = getProductReferenceByPosition(xmlDocListProducts,
												positionCategory, positionSubCategory, positionProduct,
												&pRefProduct);
		positionProduct++;
	}while(searchProduct == CATEGORY_NO_ERROR && strcmp(pRefProduct, pRef) != 0);
	
	if(searchProduct == CATEGORY_NO_ERROR && strcmp(pRefProduct, pRef) == 0){
		g_free(pRefProduct);
		return positionProduct-1;
	}else{
		fprintf(stderr, "no found the product\n");
		return -1;
	}
}

void addNewCategoryToListProducts(xmlDocPtr xmlDocListProducts, const gchar *pName){
	gint position;
	gchar *pRef = NULL;
	
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
	xmlNodePtr nodeCategory;
	
	/* generate reference */
	position = 0;
	do{
		if(pRef)
			g_free(pRef);
		pRef = g_strdup_printf("%d", position);
		position++;
	}while(getPositionCategoryByReference(xmlDocListProducts, pRef) != -1);
	
	/* create new node */
	nodeCategory = xmlNewNode(NULL, (const xmlChar*)"category");
	xmlSetProp(nodeCategory, (const xmlChar*)"name", (const xmlChar*)pName);
	xmlSetProp(nodeCategory, (const xmlChar*)"ref", (const xmlChar*)pRef);
	
	if(position == 1){
		/* first category */
		xmlAddChild(xmlDocListProducts->children, nodeCategory);
	}else{
		xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
		path = (xmlChar*)g_strdup_printf("/lda/category[%d]", position-1);
		xmlobject = xmlXPathEval(path, xmlperms_context);
		
		if(xmlobject->nodesetval->nodeNr != 0){
    	xmlAddNextSibling(xmlobject->nodesetval->nodeTab[0], nodeCategory);
  	}
  	xmlFree(path);
  	xmlXPathFreeObject(xmlobject);
  	xmlXPathFreeContext(xmlperms_context);
	}
}

void addNewSubCategoryToListProducts(xmlDocPtr xmlDocListProducts, gint positionCategory, const gchar *pName){
	gint position;
	gchar *pRef = NULL;
	
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
	xmlNodePtr nodeSubCategory;
	
	/* generate reference */
	position = 0;
	do{
		if(pRef)
			g_free(pRef);
		pRef = g_strdup_printf("%d", position);
		position++;
	}while(getPositionSubCategoryByReference(xmlDocListProducts, positionCategory, pRef) != -1);
	
	xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	/* create new node */
	nodeSubCategory = xmlNewNode(NULL, (const xmlChar*)"subcategory");
	xmlSetProp(nodeSubCategory, (const xmlChar*)"name", (const xmlChar*)pName);
	xmlSetProp(nodeSubCategory, (const xmlChar*)"ref", (const xmlChar*)pRef);
	
	if(position == 1){
		/* first subcategory */
		path = (xmlChar*)g_strdup_printf("/lda/category[%d]", positionCategory);
		xmlobject = xmlXPathEval(path, xmlperms_context);
		
		if(xmlobject->nodesetval->nodeNr != 0){
    	xmlAddChild(xmlobject->nodesetval->nodeTab[0], nodeSubCategory);
  	}
	}else{
		path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]", positionCategory, position-1);
		xmlobject = xmlXPathEval(path, xmlperms_context);
		
		if(xmlobject->nodesetval->nodeNr != 0){
    	xmlAddNextSibling(xmlobject->nodesetval->nodeTab[0], nodeSubCategory);
  	}
	}
	
	xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void addNewProductToListProducts(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, const gchar *pName, gchar *pPrice){
	gint position;
	gchar *pRef = NULL;
	
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
	xmlNodePtr nodeProduct;
	xmlNodePtr nodeProductName;
	xmlNodePtr nodeProductPrice;
	
	/* generate reference */
	position = 0;
	do{
		if(pRef)
			g_free(pRef);
		pRef = g_strdup_printf("%d", position);
		position++;
	}while(getPositionProductByReference(xmlDocListProducts, positionCategory, positionSubCategory, pRef) != -1);
	
	xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	/* create new node */
	nodeProduct = xmlNewNode(NULL, (const xmlChar*)"product");
	xmlSetProp(nodeProduct, (const xmlChar*)"ref", (const xmlChar*)pRef);
	nodeProductName = xmlNewChild(nodeProduct, NULL, (const xmlChar *)"name", (const xmlChar *)pName);
	nodeProductPrice = xmlNewChild(nodeProduct, NULL, (const xmlChar *)"price", (const xmlChar *)pPrice);
	
	if(position == 1){
		/* first product */
		path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]", positionCategory, positionSubCategory);
		xmlobject = xmlXPathEval(path, xmlperms_context);
		
		if(xmlobject->nodesetval->nodeNr != 0){
    	xmlAddChild(xmlobject->nodesetval->nodeTab[0], nodeProduct);
  	}
	}else{
		path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]/product[%d]", positionCategory, positionSubCategory, position-1);
		xmlobject = xmlXPathEval(path, xmlperms_context);
		
		if(xmlobject->nodesetval->nodeNr != 0){
    	xmlAddNextSibling(xmlobject->nodesetval->nodeTab[0], nodeProduct);
  	}
	}
	
	xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void removeCategoryFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRef){
	gint position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	position = getPositionCategoryByReference(xmlDocListProducts, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]", position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    xmlUnlinkNode(xmlobject->nodesetval->nodeTab[0]);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void removeSubCategoryFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRefCategory, gchar *pRef){
	gint positionCategory, position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	positionCategory = getPositionCategoryByReference(xmlDocListProducts, pRefCategory);
	position = getPositionSubCategoryByReference(xmlDocListProducts, positionCategory, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]", positionCategory, position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    xmlUnlinkNode(xmlobject->nodesetval->nodeTab[0]);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void removeProductFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRefCategory, gchar *pRefSubCategory, gchar *pRef){
	gint positionCategory, positionSubCategory, position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	positionCategory = getPositionCategoryByReference(xmlDocListProducts, pRefCategory);
	positionSubCategory = getPositionSubCategoryByReference(xmlDocListProducts, positionCategory, pRefSubCategory);
	position = getPositionProductByReference(xmlDocListProducts, positionCategory, positionSubCategory, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]/product[%d]", positionCategory, positionSubCategory, position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    xmlUnlinkNode(xmlobject->nodesetval->nodeTab[0]);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void modifyCategoryFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRef, const gchar *pNewName){
	gint position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	position = getPositionCategoryByReference(xmlDocListProducts, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]", position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    xmlSetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"name", (const xmlChar*)pNewName);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void ModifySubCategoryFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRefCategory, gchar *pRef, const gchar *pNewName){
	gint positionCategory, position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	positionCategory = getPositionCategoryByReference(xmlDocListProducts, pRefCategory);
	position = getPositionSubCategoryByReference(xmlDocListProducts, positionCategory, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]", positionCategory, position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    xmlSetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)"name", (const xmlChar*)pNewName);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void modifyProductFromListProducts(xmlDocPtr xmlDocListProducts, gchar *pRefCategory, gchar *pRefSubCategory, gchar *pRef, const gchar *pNewName, const gchar *pNewPrice){
	gint positionCategory, positionSubCategory, position;
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
	
	positionCategory = getPositionCategoryByReference(xmlDocListProducts, pRefCategory);
	positionSubCategory = getPositionSubCategoryByReference(xmlDocListProducts, positionCategory, pRefSubCategory);
	position = getPositionProductByReference(xmlDocListProducts, positionCategory, positionSubCategory, pRef);
	
	path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]/product[%d]", positionCategory, positionSubCategory, position);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr != 0){
    modifyProductNameFromListProducts(xmlDocListProducts, positionCategory, positionSubCategory, position, pNewName);
    modifyProductPriceFromListProducts(xmlDocListProducts, positionCategory, positionSubCategory, position, pNewPrice);
  }
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void modifyProductNameFromListProducts(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gint positionProduct, const gchar *pNewName){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]/product[%d]/name/text()", positionCategory, positionSubCategory, positionProduct);
	xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(xmlobject->nodesetval->nodeNr != 0){
    xmlNodeSetContent(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)pNewName);
  }
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void modifyProductPriceFromListProducts(xmlDocPtr xmlDocListProducts, gint positionCategory, gint positionSubCategory, gint positionProduct, const gchar *pNewPrice){
	xmlXPathContextPtr xmlperms_context = NULL;
  xmlXPathObjectPtr xmlobject;
  xmlChar *path;
  
  xmlperms_context = xmlXPathNewContext(xmlDocListProducts);
  
  path = (xmlChar*)g_strdup_printf("/lda/category[%d]/subcategory[%d]/product[%d]/price/text()", positionCategory, positionSubCategory, positionProduct);
	xmlobject = xmlXPathEval(path, xmlperms_context);
  
  if(xmlobject->nodesetval->nodeNr != 0){
    xmlNodeSetContent(xmlobject->nodesetval->nodeTab[0], (const xmlChar*)pNewPrice);
  }
  
  xmlFree(path);
  xmlXPathFreeObject(xmlobject);
  xmlXPathFreeContext(xmlperms_context);
}

void freeListProducts(xmlDocPtr xmlDocListProducts){
	xmlFreeDoc(xmlDocListProducts);
}
